@@ -162,9 +162,6 @@ rsvg_node_use_draw (RsvgNode * self, RsvgDrawingCtx *ctx,
 
 	rsvg_state_reinherit_top(ctx,  self->state, dominate);
 
-	rsvg_push_discrete_layer (ctx);
-
-
 	child = use->link;
 
 	/* If it can find nothing to draw... draw nothing */
@@ -176,9 +173,11 @@ rsvg_node_use_draw (RsvgNode * self, RsvgDrawingCtx *ctx,
 			_rsvg_affine_translate(affine, use->x, use->y);
 			_rsvg_affine_multiply(state->affine, affine, state->affine);
 
+			rsvg_push_discrete_layer (ctx);
 			rsvg_state_push(ctx);	
 			rsvg_node_draw (child, ctx, 1);
 			rsvg_state_pop(ctx);
+			rsvg_pop_discrete_layer (ctx);
 		}
 	else
 		{
@@ -195,9 +194,6 @@ rsvg_node_use_draw (RsvgNode * self, RsvgDrawingCtx *ctx,
 					(symbol->preserve_aspect_ratio, 
 					 symbol->width, symbol->height, 
 					 &width, &height, &x, &y);
-				
-				if (!symbol->overflow)
-					rsvg_add_clipping_rect (ctx, x, y, width, height);
 
 				_rsvg_affine_translate(affine, x, y);
 				_rsvg_affine_multiply(state->affine, affine, state->affine);
@@ -208,17 +204,26 @@ rsvg_node_use_draw (RsvgNode * self, RsvgDrawingCtx *ctx,
 									   -symbol->y);
 				_rsvg_affine_multiply(state->affine, affine, state->affine);
 
+				_rsvg_push_view_box(ctx, symbol->width, symbol->height);
+				rsvg_push_discrete_layer (ctx);
+				if (!state->overflow || 
+					(!state->has_overflow && child->state->overflow))
+				rsvg_add_clipping_rect (ctx, symbol->x, symbol->y,
+										symbol->width, symbol->height);
 			} else {
 				_rsvg_affine_translate(affine, use->x, use->y);
 				_rsvg_affine_multiply(state->affine, affine, state->affine);
+				rsvg_push_discrete_layer (ctx);
 			}
 
 			rsvg_state_push(ctx);
 			_rsvg_node_draw_children(child, ctx, 1);
 			rsvg_state_pop(ctx);
-
+			rsvg_pop_discrete_layer (ctx);
+			if (symbol->has_vbox)
+				_rsvg_pop_view_box(ctx);
 		}
-	rsvg_pop_discrete_layer (ctx);
+
 }	
 
 static void 
@@ -236,30 +241,38 @@ rsvg_node_svg_draw (RsvgNode * self, RsvgDrawingCtx *ctx,
 {
 	RsvgNodeSvg * sself;
 	RsvgState *state;
-	gdouble affine[6];
+	gdouble affine[6], affine_old[6], affine_new[6];
 	guint i;
+	double nx, ny, nw, nh;
 	sself = (RsvgNodeSvg *)self;
 
+	nx = _rsvg_css_normalize_length_struct(&sself->x, ctx, 'h');
+	ny = _rsvg_css_normalize_length_struct(&sself->y, ctx, 'v');
+	nw = _rsvg_css_normalize_length_struct(&sself->w, ctx, 'h');
+	nh = _rsvg_css_normalize_length_struct(&sself->h, ctx, 'v');
+
 	rsvg_state_reinherit_top(ctx, self->state, dominate);
 
-	rsvg_push_discrete_layer (ctx);
-
-	if (!sself->overflow && self->parent)
-		{
-			rsvg_add_clipping_rect(ctx, sself->x, sself->y, sself->w, sself->h);
-		}
 	state = rsvg_state_current (ctx);
 
+	for (i = 0; i < 6; i++)
+		affine_old[i] = state->affine[i];
+
 	if (sself->has_vbox && sself->hasw && sself->hash)
 		{
-			affine[0] = sself->w / sself->vbw;
+			double x = nx, y = ny, w = nw, h = nh;
+			rsvg_preserve_aspect_ratio(sself->preserve_aspect_ratio,
+									   sself->vbw, sself->vbh, 
+									   &w, &h, &x, &y);
+			affine[0] = w / sself->vbw;
 			affine[1] = 0;
 			affine[2] = 0;
-			affine[3] = sself->h / sself->vbh;
-			affine[4] = sself->x - sself->vbx * sself->w / sself->vbw;
-			affine[5] = sself->y - sself->vby * sself->h / sself->vbh;
+			affine[3] = h / sself->vbh;
+			affine[4] = x - sself->vbx * w / sself->vbw;
+			affine[5] = y - sself->vby * h / sself->vbh;
 			_rsvg_affine_multiply(state->affine, affine, 
 								  state->affine);
+			_rsvg_push_view_box(ctx, sself->vbw, sself->vbh);
 		}
 	else
 		{
@@ -267,23 +280,38 @@ rsvg_node_svg_draw (RsvgNode * self, RsvgDrawingCtx *ctx,
 			affine[1] = 0;
 			affine[2] = 0;
 			affine[3] = 1;
-			affine[4] = sself->x;
-			affine[5] = sself->y;
+			affine[4] = nx;
+			affine[5] = ny;
 			_rsvg_affine_multiply(state->affine, affine, 
 								  state->affine);
+			_rsvg_push_view_box(ctx, nw, nh);
+		}
+	for (i = 0; i < 6; i++)
+		affine_new[i] = state->affine[i];
+
+	rsvg_push_discrete_layer (ctx);
+
+	/* Bounding box addition must be AFTER the discrete layer push, 
+	   which must be AFTER the transformation happens. */
+	if (!state->overflow && self->parent)
+		{
+			for (i = 0; i < 6; i++)
+				state->affine[i] = affine_old[i];
+			rsvg_add_clipping_rect(ctx, nx, ny, nw, nh);
+			for (i = 0; i < 6; i++)
+				state->affine[i] = affine_new[i];
 		}
 
 	for (i = 0; i < self->children->len; i++)
 		{
 			rsvg_state_push(ctx);
-
 			rsvg_node_draw (g_ptr_array_index(self->children, i), 
 									 ctx, 0);
-	
 			rsvg_state_pop(ctx);
 		}			
 
 	rsvg_pop_discrete_layer (ctx);
+	_rsvg_pop_view_box(ctx);
 }
 
 static void
@@ -309,31 +337,31 @@ rsvg_node_svg_set_atts (RsvgNode * self, RsvgHandle *ctx, RsvgPropertyBag *atts)
 							ctx->height = svg->vbh;
 						}
 				}
+			if ((value = rsvg_property_bag_lookup (atts, "preserveAspectRatio")))
+				svg->preserve_aspect_ratio = rsvg_css_parse_aspect_ratio (value);			
 			if ((value = rsvg_property_bag_lookup (atts, "width")))
 				{
-					svg->w = rsvg_css_parse_normalized_length (value, ctx->dpi_x, ctx->width, 1);
-					svg->hasw = TRUE;
+					svg->w = _rsvg_css_parse_length_struct (value);
+					svg->hasw = svg->w.length > 0;
 					if (!svg->has_vbox)
-						ctx->width = svg->w; 
+						ctx->width = svg->w.length; 
 				}
 			if ((value = rsvg_property_bag_lookup (atts, "height")))
 				{
-					svg->h = rsvg_css_parse_normalized_length (value, ctx->dpi_y, ctx->height, 1);
-					svg->hash = TRUE;
+					svg->h = _rsvg_css_parse_length_struct (value);
+					svg->hash = svg->h.length > 0;
 					if (!svg->has_vbox)
-						ctx->height = svg->h;
+						ctx->height = svg->h.length;
 				}
 			if ((value = rsvg_property_bag_lookup (atts, "x")))
-				svg->x = rsvg_css_parse_normalized_length (value, ctx->dpi_x, ctx->width, 1);
+				svg->x = _rsvg_css_parse_length_struct (value);
 			if ((value = rsvg_property_bag_lookup (atts, "y")))
-				svg->y = rsvg_css_parse_normalized_length (value, ctx->dpi_y, ctx->height, 1);
+				svg->y = _rsvg_css_parse_length_struct (value);
 			if ((value = rsvg_property_bag_lookup (atts, "id")))
 				{
 					id = value;
 					rsvg_defs_register_name (ctx->defs, value, &svg->super);
 				}
-			if ((value = rsvg_property_bag_lookup (atts, "overflow")))
-				svg->overflow = rsvg_css_parse_overflow(value);
 		}
 }
 
@@ -345,13 +373,15 @@ rsvg_new_svg (void)
 	_rsvg_node_init(&svg->super);
 	svg->has_vbox = FALSE;
 	svg->preserve_aspect_ratio = RSVG_ASPECT_RATIO_XMID_YMID;
-	svg->x = 0; svg->y = 0; svg->w = -1; svg->h = -1;
+	svg->x = _rsvg_css_parse_length_struct ("0"); 
+	svg->y = _rsvg_css_parse_length_struct ("0"); 
+	svg->w = _rsvg_css_parse_length_struct ("0");
+	svg->h = _rsvg_css_parse_length_struct ("0");
 	svg->hasw = svg->hash = FALSE;
 	svg->vbx = 0; svg->vby = 0; svg->vbw = 0; svg->vbh = 0;
 	svg->super.type = RSVG_NODE_PATH;
 	svg->super.draw = rsvg_node_svg_draw;
 	svg->super.set_atts = rsvg_node_svg_set_atts;
-	svg->overflow = FALSE;
 	return &svg->super;
 }
 
@@ -438,8 +468,6 @@ rsvg_node_symbol_set_atts(RsvgNode *self, RsvgHandle *ctx, RsvgPropertyBag *atts
 				}
 			if ((value = rsvg_property_bag_lookup (atts, "preserveAspectRatio")))
 				symbol->preserve_aspect_ratio = rsvg_css_parse_aspect_ratio (value);			
-			if ((value = rsvg_property_bag_lookup (atts, "overflow")))
-				symbol->overflow = rsvg_css_parse_overflow(value);
 		}
 
 	rsvg_parse_style_attrs (ctx, self->state, "symbol", klazz, id, atts);
@@ -453,7 +481,6 @@ rsvg_new_symbol(void)
 	symbol = g_new (RsvgNodeSymbol, 1);
 	_rsvg_node_init(&symbol->super);
 	symbol->has_vbox = 0;
-	symbol->overflow = 0;
 	symbol->preserve_aspect_ratio = RSVG_ASPECT_RATIO_XMID_YMID;
 	symbol->super.type = RSVG_NODE_SYMBOL;
 	symbol->super.draw = _rsvg_node_draw_nothing;
