@@ -30,8 +30,6 @@
 #include "rsvg-paint-server.h"
 
 #include <libxml/SAX.h>
-#include <libart_lgpl/art_svp_vpath_stroke.h>
-#include <libart_lgpl/art_vpath_dash.h>
 #include <pango/pango.h>
 
 G_BEGIN_DECLS
@@ -62,10 +60,39 @@ typedef enum {
 	UNICODE_BIDI_OVERRIDE = 2	
 } UnicodeBidi;
 
+/* enums and data structures are ABI compatible with libart */
+
+typedef enum {
+  RSVG_PATH_STROKE_JOIN_MITER,
+  RSVG_PATH_STROKE_JOIN_ROUND,
+  RSVG_PATH_STROKE_JOIN_BEVEL
+} RsvgPathStrokeJoinType;
+
+typedef enum {
+  RSVG_PATH_STROKE_CAP_BUTT,
+  RSVG_PATH_STROKE_CAP_ROUND,
+  RSVG_PATH_STROKE_CAP_SQUARE
+} RsvgPathStrokeCapType;
+
+typedef struct _RsvgVpathDash RsvgVpathDash;
+
+struct _RsvgVpathDash {
+  double offset;
+  int n_dash;
+  double *dash;
+};
+
+/* end libart theft... */
+
 struct _RsvgState {
 	double affine[6];
 	double personal_affine[6];
 	
+	RsvgFilter *filter;
+	void *mask;
+	void *clip_path_ref;
+	gboolean backgroundnew;
+	guint8 adobe_blend; /* 0..11 */
 	guint8 opacity; /* 0..255 */
 	
 	RsvgPaintServer *fill;
@@ -77,11 +104,8 @@ struct _RsvgState {
 	gint clip_rule;	
 	gboolean has_clip_rule;
 
-	RsvgFilter *filter;
-	void *mask;
-	void *clip_path_ref;
-	gboolean backgroundnew;
-	guint8 adobe_blend; /* 0..11 */
+	gboolean overflow;
+	gboolean has_overflow;
 
 	RsvgPaintServer *stroke;
 	gboolean has_stroke_server;
@@ -92,9 +116,9 @@ struct _RsvgState {
 	double miter_limit;
 	gboolean has_miter_limit;
 	
-	ArtPathStrokeCapType cap;
+	RsvgPathStrokeCapType cap;
 	gboolean has_cap;
-	ArtPathStrokeJoinType join;
+	RsvgPathStrokeJoinType join;
 	gboolean has_join;
 	
 	double         font_size;
@@ -133,24 +157,18 @@ struct _RsvgState {
 	gboolean has_cond;
 	gboolean cond_true;
 
-	ArtVpathDash dash;
+	RsvgVpathDash dash;
 	gboolean has_dash;
 
 	guint32 current_color;
 	gboolean has_current_color;
 
-	RsvgDefVal * startMarker;
-	RsvgDefVal * middleMarker;
-	RsvgDefVal * endMarker;	
+	RsvgNode * startMarker;
+	RsvgNode * middleMarker;
+	RsvgNode * endMarker;	
 	gboolean has_startMarker;
 	gboolean has_middleMarker;
 	gboolean has_endMarker;	
-
-	GdkPixbuf *save_pixbuf;
-	ArtIRect underbbox;
-
-	ArtSVP * clippath;
-	int clip_path_loaded : 1;
 };
 
 RsvgState * rsvg_state_new ();
@@ -159,6 +177,7 @@ void rsvg_state_clone (RsvgState *dst, const RsvgState *src);
 void rsvg_state_inherit (RsvgState *dst, const RsvgState *src);
 void rsvg_state_reinherit (RsvgState *dst, const RsvgState *src);
 void rsvg_state_dominate (RsvgState *dst, const RsvgState *src);
+void rsvg_state_override (RsvgState *dst, const RsvgState *src);
 void rsvg_state_finalize (RsvgState *state);
 
 void rsvg_parse_style_pairs (RsvgHandle *ctx, RsvgState *state, 
@@ -175,21 +194,18 @@ void rsvg_parse_style_attrs (RsvgHandle *ctx, RsvgState *state, const char * tag
 gdouble rsvg_viewport_percentage (gdouble width, gdouble height);
 gdouble rsvg_dpi_percentage (RsvgHandle * ctx);
 
-void rsvg_pop_discrete_layer(DrawingCtx *ctx);
-void rsvg_push_discrete_layer (DrawingCtx *ctx);
-gboolean rsvg_needs_discrete_layer(RsvgState *state);
 gboolean rsvg_parse_transform (double dst[6], const char *src);
 
-RsvgState * rsvg_state_parent (DrawingCtx *ctx);
-RsvgState * rsvg_state_current (DrawingCtx *ctx);
+RsvgState * rsvg_state_parent (RsvgDrawingCtx *ctx);
+RsvgState * rsvg_state_current (RsvgDrawingCtx *ctx);
 double rsvg_state_current_font_size (RsvgHandle *ctx);
 
-void rsvg_state_clip_path_assure(DrawingCtx * ctx);
+void rsvg_state_pop(RsvgDrawingCtx * ctx);
+void rsvg_state_push(RsvgDrawingCtx * ctx);
 
-void rsvg_state_pop(DrawingCtx * ctx);
-void rsvg_state_push(DrawingCtx * ctx);
+void rsvg_state_reinherit_top(RsvgDrawingCtx * ctx, RsvgState * state, int dominate);
 
-void rsvg_state_reinherit_top(DrawingCtx * ctx, RsvgState * state, int dominate);
+void rsvg_state_reconstruct(RsvgState * state, RsvgNode * current);
 
 G_END_DECLS
 
