@@ -38,37 +38,20 @@ GQuark rsvg_error_quark (void) G_GNUC_CONST;
 
 typedef struct RsvgHandle RsvgHandle;
 
-/**
- * RsvgSizeFunc ():
- * @width: Pointer to where to set/store the width
- * @height: Pointer to where to set/store the height
- * @user_data: User data pointer
- *
- * Function to let a user of the library specify the SVG's dimensions
- * @width: the ouput width the SVG should be
- * @height: the output height the SVG should be
- * @user_data: user data
- */
-typedef void (* RsvgSizeFunc) (gint     *width,
-							   gint     *height,
-							   gpointer  user_data);
+typedef struct RsvgDimensionData RsvgDimensionData;
+struct RsvgDimensionData {
+	int width;
+	int height;
+	gdouble em, ex;
+};
 
 void        rsvg_init (void);
 void        rsvg_term (void);
 
-#ifndef RSVG_DISABLE_DEPRECATED
-void        rsvg_set_default_dpi          (double dpi);
-void        rsvg_handle_set_dpi           (RsvgHandle * handle, double dpi);
-#endif
-
-void        rsvg_set_default_dpi_x_y          (double dpi_x, double dpi_y);
-void        rsvg_handle_set_dpi_x_y           (RsvgHandle * handle, double dpi_x, double dpi_y);
+void        rsvg_set_default_dpi          (double dpi_x, double dpi_y);
+void        rsvg_handle_set_dpi           (RsvgHandle * handle, double dpi_x, double dpi_y);
 
 RsvgHandle *rsvg_handle_new               (void);
-void        rsvg_handle_set_size_callback (RsvgHandle      *handle,
-										   RsvgSizeFunc     size_func,
-										   gpointer         user_data,
-										   GDestroyNotify   user_data_destroy);
 gboolean    rsvg_handle_write             (RsvgHandle      *handle,
 										   const guchar    *buf,
 										   gsize            count,
@@ -83,6 +66,47 @@ rsvg_handle_get_base_uri (RsvgHandle *handle);
 void rsvg_handle_set_base_uri (RsvgHandle *handle,
 							   const char *base_uri);
 
+void rsvg_handle_get_dimensions(RsvgHandle * handle, RsvgDimensionData *dimension_data);
+
+/* Accessibility API */
+
+G_CONST_RETURN char *rsvg_handle_get_title         (RsvgHandle *handle);
+G_CONST_RETURN char *rsvg_handle_get_desc          (RsvgHandle *handle);
+G_CONST_RETURN char *rsvg_handle_get_metadata      (RsvgHandle *handle);
+
+RsvgHandle * rsvg_handle_new_from_data (const guint8 *data,
+										gsize data_len,
+										GError **error);
+RsvgHandle * rsvg_handle_new_from_file (const gchar *file_name,
+										GError **error);
+
+/**
+ * RsvgSizeFunc ():
+ * @width: Pointer to where to set/store the width
+ * @height: Pointer to where to set/store the height
+ * @user_data: User data pointer
+ *
+ * Function to let a user of the library specify the SVG's dimensions
+ * @width: the ouput width the SVG should be
+ * @height: the output height the SVG should be
+ * @user_data: user data
+ */
+typedef void (* RsvgSizeFunc) (gint     *width,
+							   gint     *height,
+							   gpointer  user_data);
+void        rsvg_handle_set_size_callback (RsvgHandle      *handle,
+										   RsvgSizeFunc     size_func,
+										   gpointer         user_data,
+										   GDestroyNotify   user_data_destroy);
+
+#ifndef RSVG_DISABLE_DEPRECATED
+
+/*
+ * TODO: decide whether we want to either:
+ * 1) Keep these around, and just implement it in terms of libart or cairo, whichever one we have around
+ * 2) Get rid of these completely
+ * 3) Push these into librsvg-libart solely
+ */
 
 /* Convenience API */
 
@@ -106,12 +130,7 @@ GdkPixbuf  *rsvg_pixbuf_from_file_at_zoom_with_max (const gchar  *file_name,
 													gint          max_width,
 													gint          max_height,
 													GError      **error);
-
-/* Accessibility API */
-
-G_CONST_RETURN char *rsvg_handle_get_title         (RsvgHandle *handle);
-G_CONST_RETURN char *rsvg_handle_get_desc          (RsvgHandle *handle);
-G_CONST_RETURN char *rsvg_handle_get_metadata      (RsvgHandle *handle);
+#endif /* RSVG_DISABLE_DEPRECATED */
 
 G_END_DECLS
 
